from op5helpers import *
from command import create_hosts, delete_hosts
import re

api_endpoint = 'https://'+os.environ.get('MONITOR_SERVER', 'localhost')+'/api/filter/'

def _std(**kwargs):
    """ Short proxy method for a standard request for a filter """
    r = requests.get(kwargs.get('uri', api_endpoint+kwargs.get('method', 'query')), auth=authed_user, params=kwargs.get('params', {}), verify=False)

    if "json" in r.headers['content-type'].lower():
        try:
            r.json()
        except ValueError:
            ok_(False, "Expected valid JSON, got %d:\n%s" % (r.status_code, r.text))
    status_code = kwargs.get('code', 200)
    eq_(status_code, r.status_code, "Should have got status code %d, got %d for URI:\n%s\nbody:\n%s" % (status_code, r.status_code, r.url, r.text))
    return r

@with_setup(create_hosts, delete_hosts)
def test_simple():
    r = _std(params={'query': '[hosts] (name = "blackhawk" or name = "the-up-host" or name = "bald-eagle")', 'limit': 5})
    eq_(200, r.status_code, "Should have gotten OK status code, got %d:\n%s" % (r.status_code, r.text))
    try:
        hosts = r.json()
    except ValueError:
        ok_(False, "Expected valid JSON, got %d:\n%s" % (r.status_code, r.text))
    eq_(len(hosts), 3, "The config contains five hosts, got %d:\n%s" % (len(hosts), r.text))
    host_names = sorted(map(lambda h: h['name'], hosts))
    expected_host_names = sorted([
        'blackhawk',
        'the-up-host',
        'bald-eagle'
    ])
    eq_(host_names, expected_host_names, "%s contains weird hostnames, should contain %s" % (host_names, str(expected_host_names)))

def test_bad_params():
    r = _std(params={'limit': 1}, code=400)
    ok_("Missing the parameter 'query'" in r.text, "Should have got a pretty error message, got %d:\n%s" % (r.status_code, r.text))

    def params(extra):
        # the API just complains if we leave out the query, so let's provide one
        extra['query'] = "[hosts] all"
        return extra
    r = _std(params=params({'not_existing_param': 3}), code=400)
    ok_('Unknown parameters provided' in r.text, r.text)

    r = _std(params=params({'offset': 'not_an_int'}), code=400)
    eq_(400, r.status_code, "Should have got a bad status code, got %d:\n%s" % (r.status_code, r.text))
    ok_('must be a positive number' in r.text, r.text)

@with_setup(create_hosts, delete_hosts)
def test_6443_ack_status_of_services_belonging_to_host():
    # @TODO how about quoting each parameter twice? it doesn't seem like Api_Request wants to double-decode its params, only its path =[
    r = _std(params={'query': '[services] host.name = "bald-eagle"'})
    ok_('acknowledged' in r.json()[0], 'Not asking for any specific column should include them all:\n' + r.text)

    r = _std(params={'query': '[services] host.name = "bald-eagle"', 'columns': 'description'})
    ok_('acknowledged' not in r.json()[0], 'Not asking for "acknowledged" should not include it in output:\n' + r.text)

    r = _std(params={'query': '[services] host.name = "bald-eagle"', 'columns': 'acknowledged'})
    ok_('acknowledged' in r.json()[0], 'Asking for "acknowledged" should include it in output:\n' + r.text)

def test_link_is_reachable_from_status():
    for url in [
        '/api/help/status',
        '/api/help/status/host',
        '/api/help/filter',
            ]:
        r = requests.get('https://'+os.environ.get('MONITOR_SERVER', 'localhost')+url, auth=authed_user, verify=False)
        eq_(200, r.status_code, "%s returned %d" %(url, r.status_code))
        wanted = "<a href='/api/help/filter'>Filter</a>"
        ok_(wanted in r.text, "Should have found '%s' on:\n%s\n\nGot this instead:\n%s" % (wanted, url, r.text))

def test_invalid_table_gives_useful_error_msg():
    r = _std(params={'query': '[starscream] all'}, code=400)
    ok_('Unknown table starscream' in r.text, "Wanted to find 'Unknown table starscream' in:\n\n%s" % (r.text,))

def test_invalid_columns_gives_useful_error_msg():
    r = _std(params={'query': '[hosts] all', 'columns': ['charizard']}, code=400)
    ok_("The following columns doesn't exist: charizard" in r.text, "Wanted to find 'The following columns doesn't exist: charizard' in:\n\n%s" % (r.text,))

def test_invalid_limit_gives_useful_error_msg():
    r = _std(params={'query': '[hosts] all', 'limit': 'jaws'}, code=400)
    ok_("The parameter 'limit' must be a positive number" in r.text, "Wanted to find 'The parameter 'limit' must be a positive number' in:\n\n%s" % (r.text,))

    r = _std(params={'query': '[hosts] all', 'limit': '0'}, code=400)
    ok_("The parameter 'limit' must be a positive number" in r.text, "Wanted to find 'The parameter 'limit' must be a positive number' in:\n\n%s" % (r.text,))

    r = _std(params={'query': '[hosts] all', 'limit': '-1'}, code=400)
    ok_("The parameter 'limit' must be a positive number" in r.text, "Wanted to find 'The parameter 'limit' must be a positive number' in:\n\n%s" % (r.text,))

def test_invalid_offset_gives_useful_error_msg():
    r = _std(params={'query': '[hosts] all', 'offset': 'sucka'}, code=400)
    ok_("The parameter 'offset' must be a positive number" in r.text, "Wanted to find 'The parameter 'offset' must be a positive number' in:\n\n%s" % (r.text,))

    r = _std(params={'query': '[hosts] all', 'offset': '0'}, code=400)
    ok_("The parameter 'offset' must be a positive number" in r.text, "Wanted to find 'The parameter 'offset' must be a positive number' in:\n\n%s" % (r.text,))

    r = _std(params={'query': '[hosts] all', 'offset': '-1'}, code=400)
    ok_("The parameter 'offset' must be a positive number" in r.text, "Wanted to find 'The parameter 'offset' must be a positive number' in:\n\n%s" % (r.text,))

def test_invalid_sort_gives_useful_error_msg():
    r = _std(params={'query': '[hosts] all', 'sort': 'MikeLitoris'}, code=400)
    ok_("Table 'hosts' has no column 'MikeLitoris'" in r.text, "Wanted to find 'Table 'hosts' has no column 'MikeLitoris'' in:\n\n%s" % (r.text,))

@with_setup(create_hosts, delete_hosts)
def test_limit_and_offset():
    hosts_in_fixture = '[hosts] name = "blackhawk" or name = "the-up-host" or name = "bald-eagle" or name = "swedish-chef"'
    r = _std(params={'query': hosts_in_fixture, 'columns': 'name'})
    ok_(len(r.json()) == 4, 'There should be 4 hosts in the fixture:\n%s' % (r.text,))

    r = _std(params={'query': hosts_in_fixture, 'columns': 'name', 'limit': 2})
    hosts = r.json()
    ok_(len(hosts) == 2, 'There should be 2 hosts when applying a limit to the fixture:\n%s' % (r.text,))
    eq_(hosts, [{'name': 'bald-eagle'}, {'name': 'blackhawk'}], "The first two hosts in the fixture sohuld be 'bald-eagle' and 'blackhawk'")

    r = _std(params={'query': hosts_in_fixture, 'columns': 'name', 'limit': 2, 'offset': 1})
    hosts = r.json()
    ok_(len(hosts) == 2, 'There should be 2 hosts when applying a limit to the fixture:\n%s' % (r.text,))
    eq_(hosts, [{'name': 'blackhawk'}, {'name': 'swedish-chef'}], "The first two hosts in the fixture sohuld be 'blackhawk' and 'swedish-chef', found:\n%s" % (r.text,))

@with_setup(create_hosts, delete_hosts)
def test_sort():
    hosts_in_fixture = '[hosts] name = "blackhawk" or name = "the-up-host" or name = "bald-eagle" or name = "swedish-chef"'
    before_address_sort = '[{"name":"bald-eagle","address":"maybe-not-a-proper-host.nowhere.gov"},{"name":"blackhawk","address":"not-a-proper-host.nowhere.org"},{"name":"swedish-chef","address":"mammasimammasa.gov.edu.se"},{"name":"the-up-host","address":"localhost"}]'
    after_address_sort = '[{"name":"the-up-host","address":"localhost"},{"name":"swedish-chef","address":"mammasimammasa.gov.edu.se"},{"name":"bald-eagle","address":"maybe-not-a-proper-host.nowhere.gov"},{"name":"blackhawk","address":"not-a-proper-host.nowhere.org"}]'
    after_address_sort_desc = '[{"name":"blackhawk","address":"not-a-proper-host.nowhere.org"},{"name":"bald-eagle","address":"maybe-not-a-proper-host.nowhere.gov"},{"name":"swedish-chef","address":"mammasimammasa.gov.edu.se"},{"name":"the-up-host","address":"localhost"}]'
    max_check_attempts_desc_name_asc = '[{"name":"bald-eagle","max_check_attempts":20},{"name":"blackhawk","max_check_attempts":16},{"name":"the-up-host","max_check_attempts":16},{"name":"swedish-chef","max_check_attempts":3}]'
    max_check_attempts_desc_name_desc = '[{"name":"bald-eagle","max_check_attempts":20},{"name":"the-up-host","max_check_attempts":16},{"name":"blackhawk","max_check_attempts":16},{"name":"swedish-chef","max_check_attempts":3}]'

    r = _std(uri=api_endpoint + 'query?query=%s&columns=name,address' % (hosts_in_fixture,))
    eq_(r.text, before_address_sort, "Should have the hosts sorted by name (as default), got:\n%s" % (r.text,))

    r = _std(uri=api_endpoint + 'query?query=%s&columns=name,address&sort=address' % (hosts_in_fixture,))
    eq_(r.text, after_address_sort, "Should have the hosts sorted by address, got:\n%s" % (r.text,))

    r = _std(uri=api_endpoint + 'query?query=%s&columns=name,address&sort=address ASC' % (hosts_in_fixture,))
    eq_(r.text, after_address_sort, "Should have the hosts sorted by address ASC, got:\n%s" % (r.text,))

    r = _std(uri=api_endpoint + 'query?query=%s&columns=name,address&sort=address DESC' % (hosts_in_fixture,))
    eq_(r.text, after_address_sort_desc, "Should have the hosts sorted by address DESC, got:\n%s" % (r.text,))

    r = _std(uri=api_endpoint + 'query?query=%s&columns=name,max_check_attempts&sort=max_check_attempts DESC,name ASC' % (hosts_in_fixture,))
    eq_(r.text, max_check_attempts_desc_name_asc, "Should have the hosts sorted by max_check_attempts DESC,name ASC but we got:\n%s" % (r.text,))

    r = _std(uri=api_endpoint + 'query?query=%s&columns=name,max_check_attempts&sort=max_check_attempts DESC,name DESC' % (hosts_in_fixture,))
    eq_(r.text, max_check_attempts_desc_name_desc, "Should have the hosts sorted by max_check_attempts DESC,name DESC but we got:\n%s" % (r.text,))

def test_nagstamon_status_queries():
    # we're only making sure they return 200, not bothering with fixtures
    queries = [
            'count/?query=[services]%20(state%20!=0%20or%20host.state%20!=%200)%20and%20has_been_checked%20=%201&columns=acknowledged,active_checks_enabled,current_attempt,description,host.name,host.state,host.active_checks_enabled,is_flapping,last_check,last_state_change,max_check_attempts,notifications_enabled,plugin_output,scheduled_downtime_depth,state&format=json',
            'query/?query=[services]%20(state%20!=0%20or%20host.state%20!=%200)%20and%20has_been_checked%20=%201&columns=acknowledged,active_checks_enabled,current_attempt,description,host.name,host.state,host.active_checks_enabled,is_flapping,last_check,last_state_change,max_check_attempts,notifications_enabled,plugin_output,scheduled_downtime_depth,state&format=json&limit=40',
            'count/?query=[hosts]%20state%20!=0%20and%20has_been_checked%20=%201&columns=acknowledged,active_checks_enabled,alias,current_attempt,is_flapping,last_check,last_state_change,max_check_attempts,name,notifications_enabled,plugin_output,scheduled_downtime_depth,state&format=json',
            'query/?query=[hosts]%20state%20!=0%20and%20has_been_checked%20=%201&columns=acknowledged,active_checks_enabled,alias,current_attempt,is_flapping,last_check,last_state_change,max_check_attempts,name,notifications_enabled,plugin_output,scheduled_downtime_depth,state&format=json&limit=1'
            ]
    for q in queries:
        r = _std(uri=api_endpoint + q)
        eq_(r.status_code, 200, "Should have gotten a good looking response, got %d:\n%s" % (r.status_code, r.text))

# This let's you run
# nosetests -vv -a 'nagstamon' nose/*py
# and only have these tests run
# (google for "nose attributes" or such, the version of nosetests I'm running
# doesn't support the decorator syntax, thus this clunkiness)
test_nagstamon_status_queries.nagstamon = 1
